from datetime import datetime

from django.db import models

from django_celery_beat.models import PeriodicTask


class Spider(models.Model):
    name = models.CharField(max_length=50, default=None, null=True)
    sp_name = models.CharField(max_length=20, default=None)
    media = models.IntegerField(default=None, null=True, blank=True)

    def __str__(self):
        return self.name


class JobQuerySet(models.QuerySet):
    def deleted(self):
        return self.filter(status='DEL')

    def running(self):
        return self.filter(status='RUN')

    def finished(self):
        return self.filter(status='FIN')

    def pending(self):
        return self.filter(status='PEN')


class JobManager(models.Manager):
    def get_queryset(self):
        return JobQuerySet(self.model, using=self._db)

    def deleted(self):
        return self.get_queryset().deleted()

    def running(self):
        return self.get_queryset().running()

    def finished(self):
        return self.get_queryset().finished()

    def pending(self):
        return self.get_queryset().pending()


class Job(models.Model):
    DELETE = 'DEL'
    FINISHED = 'FIN'
    RUNNING = 'RUN'
    PENDING = 'PEN'
    STATUS = (
        (DELETE, 'delete'),
        (FINISHED, 'finished'),
        (RUNNING, 'running'),
        (PENDING, 'pending')
    )
    DAY = 'DAY'
    WEEK = 'WEEK'
    MONTH = 'MONTH'
    NON = 'NON'
    INTERVAL = (
        (DAY, 'day'),
        (WEEK, 'week'),
        (MONTH, 'month'),
        (NON, 'nonstop')
    )

    name = models.CharField(max_length=50, default=None, null=True)
    job_uuid = models.CharField(max_length=50, default=None, null=True, blank=True)
    create_time = models.DateTimeField(default=datetime.now)
    start_time = models.DateTimeField(default=None, null=True, blank=True)
    end_time = models.DateTimeField(default=None, null=True, blank=True)
    status = models.CharField(max_length=3, choices=STATUS, default=PENDING)
    query = models.CharField(max_length=255, default='', null=True, blank=True)
    scrap_old_news = models.BooleanField(default=False)
    spider = models.ForeignKey(Spider, default=None, null=True)

    schedule_job = models.OneToOneField(PeriodicTask, on_delete=models.CASCADE, blank=True, null=True, default=None)
    interval = models.CharField(max_length=7, choices=INTERVAL, default=None, null=True, blank=True)
    count = models.IntegerField(default=1, blank=True, null=True)

    objects = JobManager()

    def __str__(self):
        return self.name

    def json(self):
        return {
            'id': self.pk,
            'name': self.name,
            'spider': self.spider.name,
            'create_time': self.create_time,
            'start_time': self.start_time,
            'end_time': self.end_time,
            'status': self.status
        }

    def delete(self, using=None, keep_parents=False):
        self.schedule_job.delete()
        super(Job, self).delete()


class MLData(models.Model):
    news_id = models.IntegerField()
    tag_id = models.IntegerField()

    class Meta:
        db_table = 'wp_esi_ml_data'
